@@ -3,10 +3,12 @@ import logging
 import os.path
 import sys
 
+from easel import canvas_id
 from easel import course
 from easel import files
 from easel import helpers
 from easel import helpers_yaml
+from easel import navigation_tab
 
 def cmd_login(db, args):
     hostname = args.hostname
@@ -47,6 +49,8 @@ def cmd_course(db, args):
         else:
             print("remove subcommand requires a search term for the course url"
                     " to be removed")
+    elif args.subcommand == "publish":
+        cmd_course_publish(db, args)
 
 def cmd_course_list(db):
     courses = course.find_all(db)
@@ -85,6 +89,15 @@ def cmd_course_remove(db, course_search, dry_run):
         courses[0].remove(db)
         print("removed course", courses[0].name)
 
+def cmd_course_publish(db, args):
+    if not args.course:
+        args.course = course.find_all(db)
+    else:
+        args.course = course.match_courses(db, args.course)
+
+    for course_ in args.course:
+        course.publish(course_.canvas_id, args.dry_run)
+
 def cmd_remove(db, args):
     if not args.components:
         # remove everything
@@ -186,6 +199,19 @@ def cmd_pull(db, args):
                 else:
                     remote[remote_comp.filename] = [remote_comp]
 
+        elif component_filepath in ["navigation", "navigation.yaml"]:
+            # We don't have a local version at this point so only request
+            # remote version(s). If we have a local version, the previous if
+            # clause will take care of it.
+            for course_ in args.course:
+                print(f"pulling navigation tabs from {course_.name} ({course_.canvas_id})")
+                component = navigation_tab.NavigationTabs()
+                remote_comp = component.pull(db, course_, args.dry_run)
+                if remote_comp.filename in remote:
+                    remote[remote_comp.filename].append(remote_comp)
+                else:
+                    remote[remote_comp.filename] = [remote_comp]
+
         else:
             logging.error("Cannot find file: " + component_filepath)
 
@@ -208,8 +234,9 @@ def cmd_pull(db, args):
 def cmd_push(db, args):
     if not args.components:
         # push everything
-        args.components = ["syllabus.md", "assignment_groups", "assignments",
-                "files", "pages", "quizzes", "modules"]
+        args.components = ["syllabus.md", "grading_scheme.yaml",
+                "navigation.yaml", "assignment_groups", "assignments", "files",
+                "pages", "quizzes", "modules"]
 
     if not args.course:
         args.course = course.find_all(db)
@@ -241,10 +268,34 @@ def cmd_push(db, args):
                 files.push(db, course_, component_filepath, args.hidden,
                         args.dry_run)
         else:
+            if not os.path.isfile(component_filepath):
+                logging.error("Cannot find file: " + component_filepath)
+                continue
+
             for course_ in args.course:
                 if component_filepath == "syllabus.md":
                     print(f"pushing syllabus to {course_.name} ({course_.canvas_id})")
                     course.push_syllabus(db, course_.canvas_id, args.dry_run)
+                elif component_filepath == "grading_scheme.yaml":
+                    print(f"pushing grading scheme to {course_.name} ({course_.canvas_id})")
+                    cid = canvas_id.CanvasID(component_filepath, course_.canvas_id)
+                    cid.find_id(db)
+                    if cid.canvas_id == "":
+                        # Don't try to create a scheme if it doesn't already
+                        # exist. Ideally, we update the scheme but Canvas
+                        # apparently doesn't allow for that.
+                        component = helpers_yaml.read(component_filepath)
+                        component.push(db, course_, args.dry_run)
+                        cid.find_id(db)
+                    course.update_grading_scheme(db, course_.canvas_id,
+                            cid.canvas_id, args.dry_run)
+                elif component_filepath == "navigation.yaml":
+                    print(f"pushing navigation tabs to {course_.name} ({course_.canvas_id})")
+                    navigation_tab.push(db, course_, args.dry_run)
+                elif component_filepath == "course.yaml":
+                    print(f"updating course settings for {course_.name} ({course_.canvas_id})")
+                    settings = helpers_yaml.read("course.yaml")
+                    course.update_settings(db, course_.canvas_id, settings, args.dry_run)
                 else:
                     component = helpers_yaml.read(component_filepath)
                     if component and not isinstance(component, str):
