@@ -1,5 +1,6 @@
 import logging
 import tinydb
+import yaml
 
 from easel import canvas_id
 from easel import course
@@ -29,12 +30,13 @@ class Component:
     """
 
     def __init__(self, create_path="", update_path="", db_table="",
-            canvas_wrapper="", filename=""):
+            canvas_wrapper="", filename="", yaml_order=[]):
         self.create_path = create_path
         self.update_path = update_path
         self.table = db_table
         self.canvas_wrapper = canvas_wrapper
         self.filename = filename
+        self.yaml_order = yaml_order
 
     def __iter__(self):
         if self.canvas_wrapper:
@@ -47,7 +49,7 @@ class Component:
 
     def gen_fields(self):
         ignore_these = ["create_path", "update_path", "table",
-                "canvas_wrapper", "filename"]
+                "canvas_wrapper", "filename", "yaml_order"]
         fields = vars(self)
         for field in fields.items():
             if field[0] not in ignore_these and field[1] is not None:
@@ -247,10 +249,32 @@ class Component:
         remote.filename = self.filename
         return remote
 
+    def yaml(self):
+        """Most components nicely fit the representation of a dictionary for
+        yaml. However, in case a component needs a different representation
+        (e.g., NavigationTabs is a list), override this function to produce
+        custom yaml for use in the helpers_yaml.write() function."""
+        fields = dict(self)
+        if self.canvas_wrapper:
+            # extract fields from wrapper
+            fields = fields[self.canvas_wrapper]
+
+        # reorder fields according to original ordering from the yaml file
+        # (if we have an ordering)
+        ordered = {}
+        if not self.yaml_order:
+            ordered = fields
+        for key in self.yaml_order:
+            if key in fields:
+                ordered[key] = fields[key]
+
+        return yaml.dump(ordered, sort_keys=False)
+
 def build(class_name, dictionary):
     from easel import assignment_group
     from easel import assignment
     from easel import external_tool
+    from easel import grading_scheme
     from easel import module
     from easel import module_item
     from easel import page
@@ -259,6 +283,7 @@ def build(class_name, dictionary):
             "Assignment": assignment.Assignment,
             "AssignmentGroup": assignment_group.AssignmentGroup,
             "ExternalTool": external_tool.ExternalTool,
+            "GradingScheme": grading_scheme.GradingScheme,
             "Module": module.Module,
             "ModuleItem": module_item.ModuleItem,
             "Page": page.Page,
@@ -269,22 +294,21 @@ def build(class_name, dictionary):
 def gen_filename(dir_, name):
     return dir_+"/"+name.lower().replace(' ', '_').replace('/', '-')+".yaml"
 
-def filter_fields(fields, extra_fields_to_remove=[], default_fields_to_remove=[]):
+def filter_fields(fields, fields_to_keep=[], default_fields_to_remove=[]):
     '''
     This is used to preprocess the fields of a component as returned by canvas.
-    The idea is to 1) get rid of fields that we don't care about, and 2) get
-    rid of some default values that we don't want filling up our yaml files.
+    The idea is to:
+    1. get rid of fields that we don't care about, and
+    2. get rid of fields that have default values that are usually meaningless
+       to save and we don't want them filling up our yaml files.
 
-    - extra_fields_to_remove: a list of strings where each string is a key that
-      is returned by canvas for a component but that component's easel class
-      does not know how to handle it
-    - default_fields_to_remove: a list of tuples where each tuple represents
-      the default value of a field as (field_key, default_value)
+    - fields_to_keep: a list of strings where each string is a key that is
+      a valid field for the component
+    - default_fields_to_remove: a dictionary containing the default values for
+      the corresponding keys
     '''
-    for k in extra_fields_to_remove:
-        if k in fields:
+    for k in list(fields):
+        if ((k not in fields_to_keep) or
+                (k in default_fields_to_remove and
+                fields[k] == default_fields_to_remove[k])):
             del fields[k]
-
-    for f in default_fields_to_remove:
-        if f[0] in fields and fields[f[0]] == f[1]:
-            del fields[f[0]]
