@@ -51,8 +51,10 @@ def filter_canvas_html(html):
     return html
 
 def md2html(mdtext):
-    extensions = ['fenced_code', 'codehilite', 'tables']
-    return markdown.markdown(mdtext, extensions=extensions)
+    extensions = ['fenced_code', 'codehilite', 'tables', 'attr_list']
+    config = {'codehilite': {'noclasses': True}}
+    return markdown.markdown(mdtext, extensions=extensions,
+            extension_configs=config)
 
 def write_config(hostname, token, dry_run):
     home = Path.home()
@@ -76,10 +78,6 @@ def write_config(hostname, token, dry_run):
     return True
 
 def load_db():
-    # TODO: since this just silently creates a db if it doesn't exist, it's
-    # easy to pollute the directories with tons of empty dbs. if it doesn't
-    # exist, we probably don't want to run the command anyway so should we just
-    # log and error and abort?
     return tinydb.TinyDB(".easeldb", sort_keys=True, indent=4, separators=(',', ': '))
 
 def setup_directories(dry_run):
