@@ -1,10 +1,13 @@
+import collections
 import os.path
 import yaml
 
 from easel import assignment
 from easel import assignment_group
 from easel import external_tool
+from easel import grading_scheme
 from easel import module
+from easel import navigation_tab
 from easel import page
 from easel import quiz
 from easel import quiz_question
@@ -13,7 +16,9 @@ from easel import quiz_question
 yaml.add_constructor("!Assignment", assignment.constructor)
 yaml.add_constructor("!AssignmentGroup", assignment_group.constructor)
 yaml.add_constructor("!ExternalTool", external_tool.constructor)
+yaml.add_constructor("!GradingScheme", grading_scheme.constructor)
 yaml.add_constructor("!Module", module.constructor)
+yaml.add_constructor("!NavigationTabs", navigation_tab.constructor)
 yaml.add_constructor("!Page", page.constructor)
 yaml.add_constructor("!Quiz", quiz.constructor)
 yaml.add_constructor("!QuizQuestion", quiz_question.constructor)
@@ -27,15 +32,22 @@ def read(filepath):
 def write(filepath, obj):
     if os.path.isdir(filepath):
         return None
+    # write cleaner yaml using the representer functions defined below
+    yaml.add_representer(str, str_representer)
+    yaml.representer.SafeRepresenter.add_representer(str, str_representer) # safe_dump
+
     with open(filepath, 'w') as f:
         tag = f"!{obj.__class__.__name__}"
-        fields = dict(obj)
-        if obj.canvas_wrapper:
-            fields = fields[obj.canvas_wrapper]
-        data = yaml.dump(fields)
+        data = obj.yaml()
         out = f"{tag}\n{data}"
         f.write(out)
 
+def str_representer(dumper, data):
+    '''Writes block strings for multiple lines, regular strings for single lines'''
+    if len(data.splitlines()) > 1:  # check for multiline string
+        return dumper.represent_scalar('tag:yaml.org,2002:str', data, style='|')
+    return dumper.represent_scalar('tag:yaml.org,2002:str', data)
+
 def construct_node(loader, node, class_):
     if isSequenceNode(node):
         seq = []
@@ -49,6 +61,12 @@ def construct_node(loader, node, class_):
     else:
         raise ValueError(f"Invalid yaml node type {type(node)} for {node}")
 
+def construct_ordered_mapping(loader, node):
+    '''saves the field order in the file to correctly recreate the file later'''
+    fields = loader.construct_mapping(node)
+    fields['yaml_order'] = list(fields)
+    return fields
+
 def isSequenceNode(node):
     return isinstance(node, yaml.nodes.SequenceNode)
 
